import React from 'react';
import { ReactPart, ReactPortalStore } from '../react';
import {
    PanelUpdateEvent,
    GroupPanelPartInitParameters,
    IWatermarkRenderer,
    WatermarkRendererInitParameters,
    IWatermarkPanelProps,
} from 'dockview-core';

export class ReactWatermarkPart implements IWatermarkRenderer {
    private readonly _element: HTMLElement;
    private part?: ReactPart<IWatermarkPanelProps>;
    private readonly parameters: GroupPanelPartInitParameters | undefined;

    get element(): HTMLElement {
        return this._element;
    }

    constructor(
        public readonly id: string,
        private readonly component: React.FunctionComponent<IWatermarkPanelProps>,
        private readonly reactPortalStore: ReactPortalStore
    ) {
        this._element = document.createElement('div');
        this._element.className = 'dv-react-part';
        this._element.style.height = '100%';
        this._element.style.width = '100%';
    }

    init(parameters: WatermarkRendererInitParameters): void {
        this.part = new ReactPart(
            this.element,
            this.reactPortalStore,
            this.component,
            {
                group: parameters.group,
                containerApi: parameters.containerApi,
            }
        );
    }

    focus(): void {
        // noop
    }

    update(params: PanelUpdateEvent): void {
        if (this.parameters) {
            this.parameters.params = params.params;
        }

        this.part?.update({ params: this.parameters?.params ?? {} });
    }

    layout(_width: number, _height: number): void {
        // noop - retrieval from api
    }

    dispose(): void {
        this.part?.dispose();
    }
}
